"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.validationRenderer = exports.getTaskSymbol = exports.formatDiagnosticResults = exports.getSummary = void 0;
const vscode_languageserver_types_1 = require("vscode-languageserver-types");
const chalk_1 = __importDefault(require("chalk"));
const log_symbols_1 = __importDefault(require("log-symbols"));
const elegant_spinner_1 = __importDefault(require("elegant-spinner"));
const fs_1 = require("./fs");
/** Compare the ranges and return the one that starts of finishes first */
function rangeComparator(first, second) {
    if (first.start.line < second.start.line) {
        return -1;
    }
    if (first.start.line > second.start.line) {
        return 1;
    }
    if (first.start.character < second.start.character) {
        return -1;
    }
    if (first.start.character > second.start.character) {
        return 1;
    }
    return 0;
}
/** Sort a list of digs by position */
const sortDiagnostics = (validations) => {
    // Sort the validations by line/char number
    return validations.sort((a, b) => {
        return rangeComparator(a.range, b.range);
    });
};
/** Get the text representing the line range */
function getLineRange(range) {
    return `${range.start.line + 1}:${range.start.character + 1}`;
}
/** Fix the plurality of a word */
function maybePlural(word, count) {
    return count > 1 ? `${word}s` : word;
}
/** Get the lines representing the summary of the results */
function getSummary({ errors, warnings, skipped, fileCount, duration, }) {
    return [
        '\n',
        (errors > 0 || warnings === 0) &&
            chalk_1.default.red(`${errors} ${maybePlural('error', errors)}`),
        warnings > 0 &&
            chalk_1.default.yellow(`${warnings} ${maybePlural('warning', warnings)}`),
        skipped !== undefined &&
            skipped > 0 &&
            chalk_1.default.gray(`${skipped} ${maybePlural('skipped', skipped)}`),
        chalk_1.default.gray(`in ${fileCount} ${maybePlural('file', fileCount)}`),
        chalk_1.default.gray(`took ${duration}ms`),
    ]
        .filter(Boolean)
        .join(' ');
}
exports.getSummary = getSummary;
/** Format a diag for printing on the console */
function formatDiagnostic(diag, longestLine, fName) {
    const type = diag.severity === vscode_languageserver_types_1.DiagnosticSeverity.Error
        ? chalk_1.default.red(`${log_symbols_1.default.error}  `)
        : chalk_1.default.yellow(`${log_symbols_1.default.warning}  `);
    const msg = chalk_1.default.bold(diag.message);
    const range = getLineRange(diag.range);
    return [
        `${type}`,
        range.padEnd(longestLine),
        msg,
        `${fName}:${range.padEnd(longestLine)}`,
    ].join(' ');
}
/** Format the results for printing on the console */
function formatDiagnosticResults(filePath, results, verbose = false) {
    const count = {
        errors: 0,
        warnings: 0,
    };
    const linePrefix = '  ';
    const longestLine = Math.max(...results.map((r) => getLineRange(r.range).length));
    let lines = results
        .map((diag) => {
        if (diag.severity === vscode_languageserver_types_1.DiagnosticSeverity.Error) {
            count.errors += 1;
        }
        else if (diag.severity === vscode_languageserver_types_1.DiagnosticSeverity.Warning) {
            count.warnings += 1;
        }
        if (diag.severity === vscode_languageserver_types_1.DiagnosticSeverity.Error || verbose) {
            return linePrefix + formatDiagnostic(diag, longestLine + 1, filePath);
        }
        return '';
    })
        .filter((line) => line !== '');
    if (count.errors > 0) {
        lines = ['', `${chalk_1.default.red(log_symbols_1.default.error)} ${filePath}`, ...lines, ''];
    }
    else if (verbose) {
        if (count.warnings > 0) {
            lines = [
                '',
                `${chalk_1.default.yellow(log_symbols_1.default.warning)} ${filePath}`,
                ...lines,
                '',
            ];
        }
        else {
            lines = [`${chalk_1.default.green(log_symbols_1.default.success)} ${filePath}`, ...lines];
        }
    }
    return Object.assign(Object.assign({}, count), { lines });
}
exports.formatDiagnosticResults = formatDiagnosticResults;
const spinnerState = new WeakMap();
/** Get the symbol for a given task */
const getTaskSymbol = (task) => {
    var _a;
    if (task.state === 'pending' || task.state === 'idle') {
        const spinner = (_a = spinnerState.get(task)) !== null && _a !== void 0 ? _a : (0, elegant_spinner_1.default)();
        spinnerState.set(task, spinner);
        return chalk_1.default.yellow(spinner());
    }
    if (task.state === 'completed' && task.error) {
        return log_symbols_1.default.error;
    }
    if (task.state === 'completed') {
        return chalk_1.default.yellow(log_symbols_1.default.success);
    }
    return ' ';
};
exports.getTaskSymbol = getTaskSymbol;
exports.validationRenderer = {
    onUpdate(ctx) {
        const { tasks } = ctx;
        const output = ['Validating content'];
        tasks.forEach((task) => {
            var _a, _b;
            if (task.state === 'completed' && task.output) {
                const formattedDiags = formatDiagnosticResults(((_a = task.data) === null || _a === void 0 ? void 0 : _a.file) ? (0, fs_1.normalizePath)(task.data.file) : '', sortDiagnostics(task.output), true);
                output.push(...formattedDiags.lines);
            }
            else {
                output.push(`${(0, exports.getTaskSymbol)(task)} ${((_b = task.data) === null || _b === void 0 ? void 0 : _b.file) ? (0, fs_1.normalizePath)(task.data.file) : ''}`);
            }
        });
        return output.join('\n');
    },
    onEnd(ctx) {
        const count = {
            errors: 0,
            warnings: 0,
            skipped: 0,
            fileCount: ctx.tasks.length,
            duration: ctx.duration,
        };
        ctx.tasks.forEach((t) => {
            var _a, _b;
            if (t.state === 'completed' && t.output) {
                const formattedDiags = formatDiagnosticResults((_b = (_a = t.data) === null || _a === void 0 ? void 0 : _a.file) !== null && _b !== void 0 ? _b : '', t.output, true);
                count.errors += formattedDiags.errors;
                count.warnings += formattedDiags.warnings;
            }
        });
        return [exports.validationRenderer.onUpdate(ctx), getSummary(count)].join('\n');
    },
};
